/* bzflag
 * Copyright (c) 1993 - 2003 Tim Riker
 *
 * This package is free software;  you can redistribute it and/or
 * modify it under the terms of the license found in the file
 * named LICENSE that should have accompanied this file.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* AnimMesh:
 *	classes for reading and drawing external OBJ, MD3 and  .
 */
#ifndef _ANIME_MESH_H_
#define _ANIME_MESH_H_


#include "bzfgl.h"

#include <vector>
#include <string>

#ifdef WIN32
#pragma warning( disable : 4786 )  // Disable warning message
#endif

#include <map>

#include "3dTypes.h"

typedef struct
{
  unsigned int	index;
  Vertex2D	tc;
}IndecedVert;

typedef struct
{
  IndecedVert	verts[3];
}IndexedFace;

typedef std::vector<IndexedFace>		IndexedFaceList;

typedef struct
{
  Vertex3D	faceNorm;
  Vertex3D	norms[3];
}FaceVertNorm;

typedef std::vector<Vertex3D>		VertexList;
typedef std::vector<FaceVertNorm>	FaceNormList;
typedef std::vector<FaceNormList>	FaceNormFrameList;

typedef struct
{
  Vertex3D	h,v,d,o;
}TagCS;

typedef std::vector<TagCS>	TagCSList;

typedef struct
{
  VertexList		vVerts;
  TagCSList		vTagCSs;
}AnimFrame;

typedef std::vector<AnimFrame>	FrameList;

typedef struct
{
  std::string	name;
  int		tagID;
  int		tagmateID;
}Tag;

typedef std::vector<Tag>	TagList;

class AnimModel;

class Mesh
{
public:
	Mesh();
	~Mesh();

	void draw(int iThisFrame = 1, int iNextFrame = 1, float fParam = 1);

	void smooth(float fAngle);
	void index(void);

	void reverse(void);

	IndexedFaceList		vFaces;
	FaceNormFrameList		vFaceNormalFrames;
	std::string			sName;

	Vertex3D			rScale;
	AnimModel			*pModel;
};

typedef std::vector<Mesh>	MeshList;

typedef struct
{
	int			id;
	std::string		name;
	std::string		group;
	unsigned int		start;
	unsigned int		len;
	float			speed;
}Sequence;

typedef std::vector<Sequence>	SequenceListList;

typedef struct
{
	std::string		name;
	Sequence		*seq;
	float			time;
}Event;

typedef std::vector<Event>	EventListList;

class AnimModel
{
public:
	AnimModel();
	~AnimModel();

	bool loadA3D(const char * szFile);
	bool loadU3D(const char *szPath, const char * szFile);
	bool loadMD3(const char * szFile);
	bool loadOBJ(const char * szFile);
	bool loadBZG(const char * szFile);

	bool saveA3D(const char * szFile);
	bool saveBZG(const char * szFile);

	int getMeshCount(void);
	int getMeshID(const char *szName);
	Mesh* getMesh(int iMeshID);
	
	int getNumFrames(void);
	AnimFrame* getFrame(int iFrame);

	int getNumSequences(void);
	int getSequenceID(const char *szName);
	Sequence* getSequence(int iSeqID);

	int getNumEvents(void);
	int getEventID(const char *szName);
	Event* getEvent(int iEventID);

	void drawMesh(int iMesh);
	void draw(void);

	void draw(int iThisFrame = 1, int iNextFrame = 1, float fParam = 0);

protected:
	int				iEnvironTexture;
	MeshList			vMeshes;

	SequenceListList	vSequences;
	EventListList		vEvents;

	Vertex3D			rOffset;
	Vertex3D			rRotation;
	Vertex3D			rScale;

	FrameList			vFrames;			
	TagList			vTags;

	void loadUCFile(char *szName, char* szAName, char *szDName);
};

#endif//_ANIME_MESH_H_


